/* Sortieren durch Permutation
   ---------------------------
   Es wird eine geordnete Permutation Ys der Liste Xs gesucht */

p_sortiert(Xs,Ys) :- permutiert(Xs,Ys), geordnet(Ys).


/* Eine Liste ist geordnet, wenn der Kopf X der Liste kleiner oder
   gleich dem Kopf Y der Restliste ist und die Restliste [Y|Ys] selbst 
   geordnet ist. */
   
geordnet([X]). /* Rekursionsabbruch */   
geordnet([X,Y|Ys]) :- X =< Y, geordnet([Y|Ys]).


/* Eine Liste Xs wird permutiert, indem man der Reihe nach je ein Element Z 
   der Liste lscht, die jeweilige Restliste Zs permutiert und das gelschte
   Element Z an der ersten Stelle der permutierten Restliste Ps 
   einfuegt. */
   
permutiert([X],[X]).  /* Rekursionsabbruch */   
permutiert(Xs,[Z|Ps]) :- geloescht(Z,Xs,Zs), permutiert(Zs,Ps).


/* Zunchst wird der Kopf X einer Liste gelscht; die zugehrige 
   Restliste Xs ist die bliche. 
   Als nchstes wird der Kopf Y der Restliste Xs gelscht, die verbleibende 
   Restliste hat X als Kopf und den Rest Ys als Restliste. Ys erhlt
   man, indem man das Element Y in der Liste Xs lscht. */

geloescht(X,[X|Xs],Xs).
geloescht(Y,[X|Xs],[X|Ys]) :- geloescht(Y,Xs,Ys).

/* zliste(N,G,Ls) heit: Ls ist eine Liste von N Zufallszahlen zwischen
   1 und G. */
zliste(0,G,[]).
zliste(N,G,[K|Rs]) :- N > 0, N1 is N - 1, random(G,K), zliste(N1,G,Rs).

/* Weitere Permutationsverfahren
   -----------------------------
1) Man nimmt den Kopf K einer Liste, permutiert die Restliste Rs und
   fgt anschlieend den Kopf K an allen mglichen Stellen in die
   permutierte Restliste Ys wieder ein. */
   
permutiert1([],[]).
permutiert1([K|Rs],Ps) :- permutiert1 (Rs,Ys), eingefuegt(K,Ys,Ps).

/* Das einzufuegende Element wird zunchst als Kopf vor die Liste
   gesetzt, dann der Reihe nach an den anderen Stellen eingefgt. */

eingefuegt(K,Ls,[K|Ls]).
eingefuegt(X,[K|Rs],[K|Ys]) :- eingefuegt(X,Rs,Ys).

/*
2) Man sucht alle Mengen Ps, die gleich der Menge Ls sind. Als Bedingung
   dafr gilt: Ps ist Teilmenge von Ls und Ls ist Teilmenge von Ps. */
   
permutiert2(Ls,Ps) :- teil(Ps,Ls), teil(Ls,Ps).


/* teil(Ls,Xs) heit: Ls ist Teilmenge von Xs.
   Idee: Die leere Menge ist Teilmenge jeder Menge. Lscht man den
   Kopf der Teilmenge in der Menge Xs, so enthlt die Ergebnismenge Ys
   die Restmenge Rs als Teilmenge. */
   
teil([],Xs).
teil([K|Rs], Xs) :- geloescht(K,Xs,Ys), teil(Rs,Ys).

